#version 330

uniform mat4 modelMatrix;
uniform sampler2D colorTexture;

uniform struct Light{
	vec3 position;
	vec3 intensities; //light color
} light;

in vec2 fragUV;
in vec3 fragNormal;
in vec3 fragVert;

out vec4 fragColor;

void main()
{
	//calculate normal in world coordinates
    mat3 normalMatrix = transpose(inverse(mat3(modelMatrix)));
    vec3 normal = normalize(normalMatrix * fragNormal);

	//calculate the location of this fragment (pixel) in world coordinates
    vec3 fragPosition = vec3(modelMatrix * vec4(fragVert, 1));

	//calculate the vector from this pixels surface to the light source
    vec3 surfaceToLight = normalize(light.position - fragPosition);

	//calculate the cosine of the angle of incidence (brightness)
    float brightness = dot(normal, surfaceToLight) / (length(surfaceToLight) * length(normal));
    brightness = clamp(brightness, 0, 1);
	
	
		
	vec4 lightIntensities = vec4(light.intensities, 1);

	fragColor = (brightness) * lightIntensities * texture(colorTexture, fragUV);
	
	fragColor.r = fragColor.r * .393 + fragColor.g * .769 + fragColor.b *.189;
	fragColor.g = fragColor.r * .349 + fragColor.g * .686 + fragColor.b *.168;
	fragColor.b = fragColor.r * .272 + fragColor.g * .534 + fragColor.b *.131;


}