#version 330

uniform mat4 modelMatrix;
uniform vec3 cameraPosition;

//material
uniform sampler2D colorTexture;
uniform float materialShininess;
uniform vec3 materialSpecularColor;

uniform struct Light{
	vec3 position;
	vec3 intensities; //light color
	float attenuation;
	float ambientCoefficient;
} light;

in vec2 fragUV;
in vec3 fragNormal;
in vec3 fragVert;

out vec4 fragColor;

void main()
{	
	//calculate normal in world coordinates
    mat3 normalMatrix = transpose(inverse(mat3(modelMatrix)));
    vec3 normal = normalize(normalMatrix * fragNormal);

	//calculate the location of this fragment (pixel) in world coordinates
    //surfacePos
	vec3 surfacePos = vec3(modelMatrix * vec4(fragVert, 1));
	
	vec4 surfaceColor =  texture(colorTexture, fragUV);
	
	//calculate the vector from this pixels surface to the light source
    vec3 surfaceToLight = normalize(light.position - surfacePos);
	vec3 surfaceToCamera = normalize(cameraPosition - surfacePos);
	

	//ambient
	vec3 ambient = light.ambientCoefficient * surfaceColor.rgb * light.intensities;
	
	//diffuse
    //float diffuseCoefficient = dot(normal, surfaceToLight) / (length(surfaceToLight) * length(normal));
    //oberes wird nicht mehr verwendet
	float diffuseCoefficient = dot(normal, surfaceToLight);
	diffuseCoefficient = max(0.0, diffuseCoefficient);
	vec3 diffuse = diffuseCoefficient * surfaceColor.rgb * light.intensities;

	//specular
	float specularCoefficient = 0.0;
	if(diffuseCoefficient > 0.0) {
        specularCoefficient = pow(max(0.0, dot(surfaceToCamera, reflect(-surfaceToLight, normal))), materialShininess);
	}
    vec3 specular = specularCoefficient * materialSpecularColor * light.intensities;

	
	//attenuation
	float distanceToLight = length(light.position - surfacePos);
    float attenuation = 1.0 / (1.0 + light.attenuation * pow(distanceToLight, 2));
	
	//color before gammacorrection
	vec3 linearColor = ambient + attenuation*(diffuse + specular);
	


	//intensitie added
	vec3 finalToonColor = vec3(0.0,0.0,0.0);
	
	float intensity = max(0.0, dot(normal, surfaceToLight));

	float toonFactor = 1.0;

	if(intensity > 0.95){
		toonFactor = 0.90;
	} else if(intensity > 0.50) {
		toonFactor = 0.75;
	} else if(intensity > 0.25) {
		toonFactor = 0.40;
	} else {
		toonFactor = 0.25;
	}


	vec3 textureColor = texture(colorTexture, fragUV).rgb;

	finalToonColor = finalToonColor + (toonFactor * vec3(textureColor));

	fragColor = vec4(finalToonColor, 1.0) * surfaceColor;

	
	//fragColor(after gammacorrection)
	//vec3 gamma = vec3(1.0/2.2);
	//fragColor = vec4(pow(linearColor, gamma), surfaceColor.a);

	//vec4 lightintensities = vec4(light.intensities, 1);

	//fragColor = (diffuseCoefficient) * lightintensities * surfaceColor;
	


	//vec3 color = texture(colorTexture, fragUV).rgb;	
	//fragColor = vec4(color, 1.0);

}